@@ -175,7 +175,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry *discovery.Entry, entries
 		metricName := selector.Name
 		if metricName == "" {
 			for _, lm := range selector.LabelMatchers {
-				if lm.Name == labels.MetricName && lm.Type == labels.MatchEqual {
+				if lm.Name == model.MetricNameLabel && lm.Type == labels.MatchEqual {
 					metricName = lm.Value
 					break
 				}
@@ -232,7 +232,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry *discovery.Entry, entries
 
 		labelNames := []string{}
 		for _, lm := range selector.LabelMatchers {
-			if lm.Name == labels.MetricName {
+			if lm.Name == model.MetricNameLabel {
 				continue
 			}
 			if lm.Type == labels.MatchNotEqual || lm.Type == labels.MatchNotRegexp {
@@ -479,7 +479,7 @@ func (c SeriesCheck) Check(ctx context.Context, entry *discovery.Entry, entries
 		}
 
 		for _, lm := range selector.LabelMatchers {
-			if lm.Name == labels.MetricName {
+			if lm.Name == model.MetricNameLabel {
 				continue
 			}
 			if lm.Type != labels.MatchEqual && lm.Type != labels.MatchRegexp {
@@ -495,6 +495,8 @@ func (c SeriesCheck) Check(ctx context.Context, entry *discovery.Entry, entries
 				Name:          metricName,
 				LabelMatchers: []*labels.Matcher{lm},
 			}
+			// We're checking for a specific label but we need to query using the selector name (if set).
+			// If we don't re-add the name then we'll query too much.
 			addNameSelectorIfNeeded(labelSelector, selector.LabelMatchers)
 			labelSelectorString := labelSelector.String()
 			slog.LogAttrs(ctx, slog.LevelDebug, "Checking if there are historical series matching filter", slog.String("check", c.Reporter()), slog.String("selector", labelSelectorString), slog.String("matcher", lms))
@@ -993,7 +995,7 @@ func stripLabels(selector *promParser.VectorSelector) promParser.VectorSelector
 		LabelMatchers: []*labels.Matcher{},
 	}
 	for _, lm := range selector.LabelMatchers {
-		if lm.Name == labels.MetricName {
+		if lm.Name == model.MetricNameLabel {
 			s.LabelMatchers = append(s.LabelMatchers, lm)
 			if lm.Type == labels.MatchEqual {
 				s.Name = lm.Value
@@ -1053,18 +1055,19 @@ func matchSelectorToMetric(selector *promParser.VectorSelector, metric string) (
 		// Ignore errors
 		return false, false
 	}
+
 	for _, l := range m {
 		var isMatch bool
 		for _, s := range selector.LabelMatchers {
 			if s.Type == l.Type && s.Name == l.Name && s.Value == l.Value {
-				return true, true
+				isMatch = true
 			}
 		}
 		if !isMatch {
 			return false, true
 		}
 	}
-	return false, true
+	return true, true
 }
 
 func parseRuleSet(s string) (matcher, key, value string) {
@@ -1186,10 +1189,8 @@ func avgLife(ranges []promapi.MetricTimeRange) (d time.Duration) {
 		// to have more round results
 		d += r.End.Sub(r.Start) + time.Second
 	}
-	if len(ranges) == 0 {
-		return time.Duration(0)
-	}
-	return time.Second * time.Duration(int(d.Seconds())/len(ranges))
+	// use max(1, ...) to ensure we never divide by zero
+	return time.Second * time.Duration(int(d.Seconds())/max(len(ranges), 1))
 }
 
 func oldest(ranges []promapi.MetricTimeRange) (ts time.Time) {
@@ -1214,12 +1215,6 @@ func addNameSelectorIfNeeded(selector *promParser.VectorSelector, matchers []*la
 	if selector.Name != "" {
 		return
 	}
-	for _, lm := range selector.LabelMatchers {
-		if lm.Name == model.MetricNameLabel {
-			return
-		}
-	}
-
 	for _, lm := range matchers {
 		if lm.Name == model.MetricNameLabel {
 			selector.LabelMatchers = append(selector.LabelMatchers, lm)
