@@ -81,11 +81,7 @@ func Changes(cmd CommandRunner, baseBranch string, filter PathFilter) ([]*FileCh
 		line := s.Text()
 
 		parts := strings.Split(line, "\t")
-
-		if len(parts) == 0 {
-			continue
-		}
-
+		// Split always returns at least 1 element slice.
 		if len(parts) == 1 {
 			if parts[0] != "" {
 				commit = parts[0]
@@ -201,8 +197,8 @@ func Changes(cmd CommandRunner, baseBranch string, filter PathFilter) ([]*FileCh
 		)
 
 		switch {
-		case change.Path.Before.Type != Missing && change.Path.After.Type == Symlink:
-			slog.LogAttrs(context.Background(), slog.LevelDebug, "File was turned into a symlink", slog.String("path", change.Path.After.Name))
+		case change.Path.Before.Type != Missing && change.Path.Before.Type != Symlink && change.Path.After.Type == Symlink:
+			slog.LogAttrs(context.Background(), slog.LevelDebug, "Path was turned into a symlink", slog.String("path", change.Path.After.Name))
 			change.Body.ModifiedLines = CountLines(change.Body.After)
 		case change.Path.Before.Type != Missing && change.Path.After.Type != Missing && change.Path.After.Type != Symlink:
 			change.Body.ModifiedLines, err = getModifiedLines(cmd, change.Commits, change.Path.After.EffectivePath(), lastCommit, change.Body.Before, change.Body.After)
@@ -351,8 +347,8 @@ func getTypeForPath(cmd CommandRunner, commit, fpath string) PathType {
 }
 
 // recursively find the final target of a symlink.
-func resolveSymlinkTarget(cmd CommandRunner, commit, fpath string, typ PathType) string {
-	if typ != Symlink {
+func resolveSymlinkTarget(cmd CommandRunner, commit, fpath string, pathType PathType) string {
+	if pathType != Symlink {
 		return fpath
 	}
 	raw := string(getContentAtCommit(cmd, commit, fpath))
