@@ -8,6 +8,7 @@ import (
 	"strconv"
 	"strings"
 
+	"github.com/prometheus/common/model"
 	"github.com/prometheus/prometheus/model/labels"
 	promParser "github.com/prometheus/prometheus/promql/parser"
 	"github.com/prometheus/prometheus/promql/parser/posrange"
@@ -439,17 +440,17 @@ func (s *Source) checkJoinedLabels(expr string, n *promParser.BinaryExpr, dst So
 	return dead
 }
 
-func (s *Source) useLabelsNotExcluded(exluded []string) {
+func (s *Source) useLabelsNotExcluded(excluded []string) {
 	// Iterating over a map can yield labels in different order each time
 	// so append labels to an extra slice, sort it, and then append the
-	// sorted reults to UsedLabels.
+	// sorted results to UsedLabels.
 	// Without this tests might show a diff sometimes.
 	toAdd := make([]string, 0, len(s.Labels))
 	for name, lt := range s.Labels {
 		if lt.Kind == ImpossibleLabel {
 			continue
 		}
-		if !slices.Contains(exluded, name) {
+		if !slices.Contains(excluded, name) {
 			toAdd = appendToSlice(toAdd, name)
 		}
 	}
@@ -536,8 +537,8 @@ func walkNode(expr string, node promParser.Node) (src []Source) {
 	case *promParser.UnaryExpr:
 		src = append(src, walkNode(expr, n.Expr)...)
 
-	case *promParser.StepInvariantExpr:
-		// Not possible to get this from the parser.
+	// case *promParser.StepInvariantExpr:
+	// Not possible to get this from the parser.
 
 	case *promParser.VectorSelector:
 		var s Source
@@ -563,9 +564,6 @@ func walkNode(expr string, node promParser.Node) (src []Source) {
 		}
 		s.Position = n.PosRange
 		src = append(src, s)
-
-	default:
-		// unhandled type
 	}
 	return src
 }
@@ -585,7 +583,7 @@ func labelsFromSelectors(matches []labels.MatchType, selector *promParser.Vector
 	}
 	// Any label used in positive filters is gurnateed to be present.
 	for _, lm := range selector.LabelMatchers {
-		if lm.Name == labels.MetricName {
+		if lm.Name == model.MetricNameLabel {
 			continue
 		}
 		if !slices.Contains(matches, lm.Type) {
@@ -598,7 +596,7 @@ func labelsFromSelectors(matches []labels.MatchType, selector *promParser.Vector
 
 func labelsWithEmptyValueSelector(selector *promParser.VectorSelector) (names []string) {
 	for _, lm := range selector.LabelMatchers {
-		if lm.Name == labels.MetricName {
+		if lm.Name == model.MetricNameLabel {
 			continue
 		}
 		if lm.Type == labels.MatchEqual && lm.Value == "" {
@@ -634,8 +632,8 @@ func walkAggregation(expr string, n *promParser.AggregateExpr) (src []Source) {
 				n.PosRange,
 				n.Param.(*promParser.StringLiteral).Val,
 			)
-			if n.Without || !slices.Contains(n.Grouping, labels.MetricName) {
-				s.excludeLabel("Aggregation removes metric name.", n.PosRange, labels.MetricName)
+			if n.Without || !slices.Contains(n.Grouping, model.MetricNameLabel) {
+				s.excludeLabel("Aggregation removes metric name.", n.PosRange, model.MetricNameLabel)
 			}
 			src = append(src, s)
 		}
@@ -659,8 +657,8 @@ func walkAggregation(expr string, n *promParser.AggregateExpr) (src []Source) {
 				Node:      n,
 				Arguments: args,
 			})
-			if n.Without || !slices.Contains(n.Grouping, labels.MetricName) {
-				s.excludeLabel("Aggregation removes metric name.", n.PosRange, labels.MetricName)
+			if n.Without || !slices.Contains(n.Grouping, model.MetricNameLabel) {
+				s.excludeLabel("Aggregation removes metric name.", n.PosRange, model.MetricNameLabel)
 			}
 			src = append(src, s)
 		}
@@ -1131,7 +1129,7 @@ func parseBinOps(expr string, n *promParser.BinaryExpr) (src []Source) {
 				})
 			}
 			ls.DeadLabels = append(ls.DeadLabels, ls.checkJoinedLabels(expr, n, ls)...)
-			ls.excludeLabel("Binary operation between two vectors removes metric names.", pos, labels.MetricName)
+			ls.excludeLabel("Binary operation between two vectors removes metric names.", pos, model.MetricNameLabel)
 			ls.IsConditional, ls.ReturnInfo.IsReturnBool = checkConditions(ls, n.Op, n.ReturnBool)
 			src = append(src, ls)
 		}
@@ -1195,7 +1193,7 @@ func parseBinOps(expr string, n *promParser.BinaryExpr) (src []Source) {
 					IsOn:           n.VectorMatching.On,
 				})
 			}
-			rs.excludeLabel("Binary operation between two vectors removes metric names.", pos, labels.MetricName)
+			rs.excludeLabel("Binary operation between two vectors removes metric names.", pos, model.MetricNameLabel)
 			rs.IsConditional, rs.ReturnInfo.IsReturnBool = checkConditions(rs, n.Op, n.ReturnBool)
 			src = append(src, rs)
 		}
@@ -1256,7 +1254,7 @@ func parseBinOps(expr string, n *promParser.BinaryExpr) (src []Source) {
 					IsOn:           n.VectorMatching.On,
 				})
 			}
-			ls.excludeLabel("Binary operation between two vectors removes metric names.", pos, labels.MetricName)
+			ls.excludeLabel("Binary operation between two vectors removes metric names.", pos, model.MetricNameLabel)
 			ls.IsConditional, ls.ReturnInfo.IsReturnBool = checkConditions(ls, n.Op, n.ReturnBool)
 			src = append(src, ls)
 		}
@@ -1359,7 +1357,7 @@ func parseBinOps(expr string, n *promParser.BinaryExpr) (src []Source) {
 				// If LHS can NOT be empty then RHS is dead code.
 				if !lhsCanBeEmpty {
 					rs.DeadInfo = &DeadInfo{
-						Reason:   "The left hand side always returs something and so the right hand side is never used.",
+						Reason:   "The left hand side always returns something and so the right hand side is never used.",
 						Fragment: rs.Position,
 					}
 				}
@@ -1406,6 +1404,9 @@ func canJoin(ls, rs Source, vm *promParser.VectorMatching) (bool, string, posran
 			if l.Kind != GuaranteedLabel {
 				continue
 			}
+			if slices.Contains(vm.MatchingLabels, name) {
+				continue
+			}
 			if ls.CanHaveLabel(name) && !rs.CanHaveLabel(name) {
 				reason, fragment := rs.LabelExcludeReason(name)
 				return false, fmt.Sprintf("The %s hand side will never be matched because it doesn't have the `%s` label while the left hand side will. %s",
