@@ -7,6 +7,8 @@ import (
 	"io"
 	"log/slog"
 	"strconv"
+
+	"github.com/cloudflare/pint/internal/checks"
 )
 
 func NewCheckStyleReporter(output io.Writer) CheckStyleReporter {
@@ -19,14 +21,8 @@ type CheckStyleReporter struct {
 	output io.Writer
 }
 
-type checkstyleReport map[string][]Report
-
-func createCheckstyleReport(summary Summary) checkstyleReport {
-	x := make(checkstyleReport)
-	for _, report := range summary.reports {
-		x[report.Path.Name] = append(x[report.Path.Name], report)
-	}
-	return x
+type checkstyleReport struct {
+	reportsPerPath [][]Report
 }
 
 func (d checkstyleReport) MarshalXML(e *xml.Encoder, _ xml.StartElement) (err error) {
@@ -42,14 +38,17 @@ func (d checkstyleReport) MarshalXML(e *xml.Encoder, _ xml.StartElement) (err er
 	if err != nil {
 		return err
 	}
-	for dir, reports := range d {
+	for _, reports := range d.reportsPerPath {
+		if len(reports) == 0 {
+			continue
+		}
 		if err = e.EncodeToken(
 			xml.StartElement{
 				Name: xml.Name{Local: "file"},
 				Attr: []xml.Attr{
 					{
 						Name:  xml.Name{Local: "name"},
-						Value: dir,
+						Value: reports[0].Path.Name,
 					},
 				},
 			}); err != nil {
@@ -72,6 +71,19 @@ func (r Report) MarshalXML(e *xml.Encoder, _ xml.StartElement) (err error) {
 	if r.Problem.Details != "" {
 		msg += "\n" + r.Problem.Details
 	}
+
+	var checkstyleSeverity string
+	switch r.Problem.Severity {
+	case checks.Information:
+		checkstyleSeverity = "info"
+	case checks.Warning:
+		checkstyleSeverity = "warning"
+	case checks.Bug:
+		checkstyleSeverity = "error"
+	case checks.Fatal:
+		checkstyleSeverity = "error"
+	}
+
 	startel := xml.StartElement{
 		Name: xml.Name{Local: "error"},
 		Attr: []xml.Attr{
@@ -81,7 +93,7 @@ func (r Report) MarshalXML(e *xml.Encoder, _ xml.StartElement) (err error) {
 			},
 			{
 				Name:  xml.Name{Local: "severity"},
-				Value: r.Problem.Severity.String(),
+				Value: checkstyleSeverity,
 			},
 			{
 				Name:  xml.Name{Local: "message"},
@@ -100,8 +112,8 @@ func (r Report) MarshalXML(e *xml.Encoder, _ xml.StartElement) (err error) {
 }
 
 func (cs CheckStyleReporter) Submit(ctx context.Context, summary Summary) error {
-	checkstyleReport := createCheckstyleReport(summary)
-	xmlString, err := xml.MarshalIndent(checkstyleReport, "", "  ")
+	report := checkstyleReport{reportsPerPath: summary.ReportsPerPath()}
+	xmlString, err := xml.MarshalIndent(report, "", "  ")
 	if err != nil {
 		slog.LogAttrs(ctx, slog.LevelError, "Failed to marshal checkstyle report", slog.Any("err", err))
 		return err
