@@ -19,6 +19,7 @@ package gitlab
 import (
 	"bytes"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"io"
 	"net/http"
@@ -116,6 +117,7 @@ type Group struct {
 	MembershipLock                  bool                       `json:"membership_lock"`
 	Visibility                      VisibilityValue            `json:"visibility"`
 	LFSEnabled                      bool                       `json:"lfs_enabled"`
+	MaxArtifactsSize                int64                      `json:"max_artifacts_size"`
 	DefaultBranch                   string                     `json:"default_branch"`
 	DefaultBranchProtectionDefaults *BranchProtectionDefaults  `json:"default_branch_protection_defaults"`
 	AvatarURL                       string                     `json:"avatar_url"`
@@ -152,6 +154,10 @@ type Group struct {
 	AllowedEmailDomainsList         string                     `json:"allowed_email_domains_list"`
 	WikiAccessLevel                 AccessControlValue         `json:"wiki_access_level"`
 
+	OnlyAllowMergeIfPipelineSucceeds          bool `json:"only_allow_merge_if_pipeline_succeeds"`
+	AllowMergeOnSkippedPipeline               bool `json:"allow_merge_on_skipped_pipeline"`
+	OnlyAllowMergeIfAllDiscussionsAreResolved bool `json:"only_allow_merge_if_all_discussions_are_resolved"`
+
 	// Deprecated: will be removed in v5 of the API, use ListGroupProjects instead
 	Projects []*Project `json:"projects"`
 
@@ -257,18 +263,11 @@ type ListGroupsOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-groups
 func (s *GroupsService) ListGroups(opt *ListGroupsOptions, options ...RequestOptionFunc) ([]*Group, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "groups", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var gs []*Group
-	resp, err := s.client.Do(req, &gs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gs, resp, nil
+	return do[[]*Group](s.client,
+		withPath("groups"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // ListSubGroupsOptions represents the available ListSubGroups() options.
@@ -282,24 +281,11 @@ type ListSubGroupsOptions ListGroupsOptions
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-subgroups
 func (s *GroupsService) ListSubGroups(gid any, opt *ListSubGroupsOptions, options ...RequestOptionFunc) ([]*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/subgroups", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var gs []*Group
-	resp, err := s.client.Do(req, &gs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gs, resp, nil
+	return do[[]*Group](s.client,
+		withPath("groups/%s/subgroups", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // ListDescendantGroupsOptions represents the available ListDescendantGroups()
@@ -314,32 +300,20 @@ type ListDescendantGroupsOptions ListGroupsOptions
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-descendant-groups
 func (s *GroupsService) ListDescendantGroups(gid any, opt *ListDescendantGroupsOptions, options ...RequestOptionFunc) ([]*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/descendant_groups", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var gs []*Group
-	resp, err := s.client.Do(req, &gs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gs, resp, nil
+	return do[[]*Group](s.client,
+		withPath("groups/%s/descendant_groups", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
-// ListGroupProjectsOptions represents the available ListGroup() options.
+// ListGroupProjectsOptions represents the available ListGroupProjects() options.
 //
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-projects
 type ListGroupProjectsOptions struct {
 	ListOptions
+	Active                   *bool             `url:"active,omitempty" json:"active,omitempty"`
 	Archived                 *bool             `url:"archived,omitempty" json:"archived,omitempty"`
 	IncludeSubGroups         *bool             `url:"include_subgroups,omitempty" json:"include_subgroups,omitempty"`
 	MinAccessLevel           *AccessLevelValue `url:"min_access_level,omitempty" json:"min_access_level,omitempty"`
@@ -363,24 +337,11 @@ type ListGroupProjectsOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-projects
 func (s *GroupsService) ListGroupProjects(gid any, opt *ListGroupProjectsOptions, options ...RequestOptionFunc) ([]*Project, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/projects", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var ps []*Project
-	resp, err := s.client.Do(req, &ps)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return ps, resp, nil
+	return do[[]*Project](s.client,
+		withPath("groups/%s/projects", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // GetGroupOptions represents the available GetGroup() options.
@@ -398,24 +359,11 @@ type GetGroupOptions struct {
 //
 // GitLab API docs: https://docs.gitlab.com/api/groups/#get-a-single-group
 func (s *GroupsService) GetGroup(gid any, opt *GetGroupOptions, options ...RequestOptionFunc) (*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	g := new(Group)
-	resp, err := s.client.Do(req, g)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return g, resp, nil
+	return do[*Group](s.client,
+		withPath("groups/%s", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // DownloadAvatar downloads a group avatar.
@@ -540,7 +488,7 @@ func (s *GroupsService) CreateGroup(opt *CreateGroupOptions, options ...RequestO
 		// since the Avatar is provided, check allowed_to_push and
 		// allowed_to_merge access levels and error if multiples are provided
 		if opt.DefaultBranchProtectionDefaults != nil && (len(*opt.DefaultBranchProtectionDefaults.AllowedToMerge) > 1 || len(*opt.DefaultBranchProtectionDefaults.AllowedToPush) > 1) {
-			return nil, nil, fmt.Errorf("multiple access levels for allowed_to_merge or allowed_to_push are not permitted when an Avatar is also specified as it will result in unexpected behavior")
+			return nil, nil, errors.New("multiple access levels for allowed_to_merge or allowed_to_push are not permitted when an Avatar is also specified as it will result in unexpected behavior")
 		}
 		req, err = s.client.UploadRequest(
 			http.MethodPost,
@@ -571,28 +519,11 @@ func (s *GroupsService) CreateGroup(opt *CreateGroupOptions, options ...RequestO
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#transfer-a-project-to-a-group
 func (s *GroupsService) TransferGroup(gid any, pid any, options ...RequestOptionFunc) (*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	project, err := parseID(pid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/projects/%s", PathEscape(group), PathEscape(project))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	g := new(Group)
-	resp, err := s.client.Do(req, g)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return g, resp, nil
+	return do[*Group](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/projects/%s", GroupID{gid}, ProjectID{pid}),
+		withRequestOpts(options...),
+	)
 }
 
 // TransferSubGroupOptions represents the available TransferSubGroup() options.
@@ -609,24 +540,12 @@ type TransferSubGroupOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#transfer-a-group
 func (s *GroupsService) TransferSubGroup(gid any, opt *TransferSubGroupOptions, options ...RequestOptionFunc) (*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/transfer", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	g := new(Group)
-	resp, err := s.client.Do(req, g)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return g, resp, nil
+	return do[*Group](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/transfer", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // UpdateGroupOptions represents the available UpdateGroup() options.
@@ -649,6 +568,7 @@ type UpdateGroupOptions struct {
 	EmailsEnabled                        *bool                                   `url:"emails_enabled,omitempty" json:"emails_enabled,omitempty"`
 	MentionsDisabled                     *bool                                   `url:"mentions_disabled,omitempty" json:"mentions_disabled,omitempty"`
 	LFSEnabled                           *bool                                   `url:"lfs_enabled,omitempty" json:"lfs_enabled,omitempty"`
+	MaxArtifactsSize                     *int64                                  `url:"max_artifacts_size,omitempty" json:"max_artifacts_size,omitempty"`
 	RequestAccessEnabled                 *bool                                   `url:"request_access_enabled,omitempty" json:"request_access_enabled,omitempty"`
 	DefaultBranchProtectionDefaults      *DefaultBranchProtectionDefaultsOptions `url:"default_branch_protection_defaults,omitempty" json:"default_branch_protection_defaults,omitempty"`
 	FileTemplateProjectID                *int64                                  `url:"file_template_project_id,omitempty" json:"file_template_project_id,omitempty"`
@@ -661,6 +581,10 @@ type UpdateGroupOptions struct {
 	AllowedEmailDomainsList              *string                                 `url:"allowed_email_domains_list,omitempty" json:"allowed_email_domains_list,omitempty"`
 	WikiAccessLevel                      *AccessControlValue                     `url:"wiki_access_level,omitempty" json:"wiki_access_level,omitempty"`
 
+	OnlyAllowMergeIfPipelineSucceeds          *bool `url:"only_allow_merge_if_pipeline_succeeds,omitempty" json:"only_allow_merge_if_pipeline_succeeds,omitempty"`
+	AllowMergeOnSkippedPipeline               *bool `url:"allow_merge_on_skipped_pipeline,omitempty" json:"allow_merge_on_skipped_pipeline,omitempty"`
+	OnlyAllowMergeIfAllDiscussionsAreResolved *bool `url:"only_allow_merge_if_all_discussions_are_resolved,omitempty" json:"only_allow_merge_if_all_discussions_are_resolved,omitempty"`
+
 	// Deprecated: Use EmailsEnabled instead
 	EmailsDisabled *bool `url:"emails_disabled,omitempty" json:"emails_disabled,omitempty"`
 
@@ -693,7 +617,7 @@ func (s *GroupsService) UpdateGroup(gid any, opt *UpdateGroupOptions, options ..
 		// since the Avatar is provided, check allowed_to_push and
 		// allowed_to_merge access levels and error if multiples are provided
 		if opt.DefaultBranchProtectionDefaults != nil && (len(*opt.DefaultBranchProtectionDefaults.AllowedToMerge) > 1 || len(*opt.DefaultBranchProtectionDefaults.AllowedToPush) > 1) {
-			return nil, nil, fmt.Errorf("multiple access levels for allowed_to_merge or allowed_to_push are not permitted when an Avatar is also specified as it will result in unexpected behavior")
+			return nil, nil, errors.New("multiple access levels for allowed_to_merge or allowed_to_push are not permitted when an Avatar is also specified as it will result in unexpected behavior")
 		}
 		req, err = s.client.UploadRequest(
 			http.MethodPut,
@@ -763,18 +687,13 @@ type DeleteGroupOptions struct {
 //
 // GitLab API docs: https://docs.gitlab.com/api/groups/#delete-a-group
 func (s *GroupsService) DeleteGroup(gid any, opt *DeleteGroupOptions, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf("groups/%s", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, opt, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // RestoreGroup restores a previously deleted group
@@ -782,24 +701,11 @@ func (s *GroupsService) DeleteGroup(gid any, opt *DeleteGroupOptions, options ..
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#restore-a-group-marked-for-deletion
 func (s *GroupsService) RestoreGroup(gid any, options ...RequestOptionFunc) (*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/restore", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	g := new(Group)
-	resp, err := s.client.Do(req, g)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return g, resp, nil
+	return do[*Group](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/restore", GroupID{gid}),
+		withRequestOpts(options...),
+	)
 }
 
 // SearchGroup get all groups that match your string in their name or path.
@@ -811,18 +717,11 @@ func (s *GroupsService) SearchGroup(query string, options ...RequestOptionFunc)
 	}
 	q.Search = query
 
-	req, err := s.client.NewRequest(http.MethodGet, "groups", &q, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var gs []*Group
-	resp, err := s.client.Do(req, &gs)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gs, resp, nil
+	return do[[]*Group](s.client,
+		withPath("groups"),
+		withAPIOpts(&q),
+		withRequestOpts(options...),
+	)
 }
 
 // ListProvisionedUsersOptions represents the available ListProvisionedUsers()
@@ -845,24 +744,11 @@ type ListProvisionedUsersOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-provisioned-users
 func (s *GroupsService) ListProvisionedUsers(gid any, opt *ListProvisionedUsersOptions, options ...RequestOptionFunc) ([]*User, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/provisioned_users", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var us []*User
-	resp, err := s.client.Do(req, &us)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return us, resp, nil
+	return do[[]*User](s.client,
+		withPath("groups/%s/provisioned_users", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // ListGroupLDAPLinks lists the group's LDAP links. Available only for users who
@@ -871,24 +757,10 @@ func (s *GroupsService) ListProvisionedUsers(gid any, opt *ListProvisionedUsersO
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_ldap_links/#list-ldap-group-links
 func (s *GroupsService) ListGroupLDAPLinks(gid any, options ...RequestOptionFunc) ([]*LDAPGroupLink, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/ldap_group_links", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var gls []*LDAPGroupLink
-	resp, err := s.client.Do(req, &gls)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gls, resp, nil
+	return do[[]*LDAPGroupLink](s.client,
+		withPath("groups/%s/ldap_group_links", GroupID{gid}),
+		withRequestOpts(options...),
+	)
 }
 
 // AddGroupLDAPLinkOptions represents the available AddGroupLDAPLink() options.
@@ -909,24 +781,12 @@ type AddGroupLDAPLinkOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_ldap_links/#add-an-ldap-group-link-with-cn-or-filter
 func (s *GroupsService) AddGroupLDAPLink(gid any, opt *AddGroupLDAPLinkOptions, options ...RequestOptionFunc) (*LDAPGroupLink, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/ldap_group_links", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	gl := new(LDAPGroupLink)
-	resp, err := s.client.Do(req, gl)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gl, resp, nil
+	return do[*LDAPGroupLink](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/ldap_group_links", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // DeleteGroupLDAPLink deletes a group LDAP link. Available only for users who
@@ -936,18 +796,12 @@ func (s *GroupsService) AddGroupLDAPLink(gid any, opt *AddGroupLDAPLinkOptions,
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_ldap_links/#delete-an-ldap-group-link-deprecated
 func (s *GroupsService) DeleteGroupLDAPLink(gid any, cn string, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf("groups/%s/ldap_group_links/%s", PathEscape(group), PathEscape(cn))
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s/ldap_group_links/%s", GroupID{gid}, cn),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // DeleteGroupLDAPLinkWithCNOrFilterOptions represents the available DeleteGroupLDAPLinkWithCNOrFilter() options.
@@ -966,18 +820,13 @@ type DeleteGroupLDAPLinkWithCNOrFilterOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_ldap_links/#delete-an-ldap-group-link-with-cn-or-filter
 func (s *GroupsService) DeleteGroupLDAPLinkWithCNOrFilter(gid any, opts *DeleteGroupLDAPLinkWithCNOrFilterOptions, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf("groups/%s/ldap_group_links", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, opts, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s/ldap_group_links", GroupID{gid}),
+		withAPIOpts(opts),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // DeleteGroupLDAPLinkForProvider deletes a group LDAP link from a specific
@@ -986,23 +835,12 @@ func (s *GroupsService) DeleteGroupLDAPLinkWithCNOrFilter(gid any, opts *DeleteG
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_ldap_links/#delete-an-ldap-group-link-deprecated
 func (s *GroupsService) DeleteGroupLDAPLinkForProvider(gid any, provider, cn string, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf(
-		"groups/%s/ldap_group_links/%s/%s",
-		PathEscape(group),
-		PathEscape(provider),
-		PathEscape(cn),
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s/ldap_group_links/%s/%s", GroupID{gid}, provider, cn),
+		withRequestOpts(options...),
 	)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	return resp, err
 }
 
 // ListGroupSAMLLinks lists the group's SAML links. Available only for users who
@@ -1011,24 +849,10 @@ func (s *GroupsService) DeleteGroupLDAPLinkForProvider(gid any, provider, cn str
 // GitLab API docs:
 // https://docs.gitlab.com/api/saml/#list-saml-group-links
 func (s *GroupsService) ListGroupSAMLLinks(gid any, options ...RequestOptionFunc) ([]*SAMLGroupLink, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/saml_group_links", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var gl []*SAMLGroupLink
-	resp, err := s.client.Do(req, &gl)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gl, resp, nil
+	return do[[]*SAMLGroupLink](s.client,
+		withPath("groups/%s/saml_group_links", GroupID{gid}),
+		withRequestOpts(options...),
+	)
 }
 
 // ListGroupSharedProjectsOptions represents the available ListGroupSharedProjects() options.
@@ -1055,24 +879,11 @@ type ListGroupSharedProjectsOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#list-shared-projects
 func (s *GroupsService) ListGroupSharedProjects(gid any, opt *ListGroupSharedProjectsOptions, options ...RequestOptionFunc) ([]*Project, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/projects/shared", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var p []*Project
-	resp, err := s.client.Do(req, &p)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return p, resp, nil
+	return do[[]*Project](s.client,
+		withPath("groups/%s/projects/shared", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // GetGroupSAMLLink get a specific group SAML link. Available only for users who
@@ -1081,24 +892,10 @@ func (s *GroupsService) ListGroupSharedProjects(gid any, opt *ListGroupSharedPro
 // GitLab API docs:
 // https://docs.gitlab.com/api/saml/#get-a-saml-group-link
 func (s *GroupsService) GetGroupSAMLLink(gid any, samlGroupName string, options ...RequestOptionFunc) (*SAMLGroupLink, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/saml_group_links/%s", PathEscape(group), PathEscape(samlGroupName))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	gl := new(SAMLGroupLink)
-	resp, err := s.client.Do(req, &gl)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gl, resp, nil
+	return do[*SAMLGroupLink](s.client,
+		withPath("groups/%s/saml_group_links/%s", GroupID{gid}, samlGroupName),
+		withRequestOpts(options...),
+	)
 }
 
 // AddGroupSAMLLinkOptions represents the available AddGroupSAMLLink() options.
@@ -1117,24 +914,12 @@ type AddGroupSAMLLinkOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/saml/#add-a-saml-group-link
 func (s *GroupsService) AddGroupSAMLLink(gid any, opt *AddGroupSAMLLinkOptions, options ...RequestOptionFunc) (*SAMLGroupLink, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/saml_group_links", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	gl := new(SAMLGroupLink)
-	resp, err := s.client.Do(req, &gl)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gl, resp, nil
+	return do[*SAMLGroupLink](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/saml_group_links", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // DeleteGroupSAMLLink deletes a group SAML link. Available only for users who
@@ -1143,18 +928,12 @@ func (s *GroupsService) AddGroupSAMLLink(gid any, opt *AddGroupSAMLLinkOptions,
 // GitLab API docs:
 // https://docs.gitlab.com/api/saml/#delete-a-saml-group-link
 func (s *GroupsService) DeleteGroupSAMLLink(gid any, samlGroupName string, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf("groups/%s/saml_group_links/%s", PathEscape(group), PathEscape(samlGroupName))
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s/saml_group_links/%s", GroupID{gid}, samlGroupName),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // ShareGroupWithGroupOptions represents the available ShareGroupWithGroup() options.
@@ -1173,24 +952,12 @@ type ShareGroupWithGroupOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#create-a-link-to-share-a-group-with-another-group
 func (s *GroupsService) ShareGroupWithGroup(gid any, opt *ShareGroupWithGroupOptions, options ...RequestOptionFunc) (*Group, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/share", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	g := new(Group)
-	resp, err := s.client.Do(req, g)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return g, resp, nil
+	return do[*Group](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/share", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // UnshareGroupFromGroup unshares a group from another group.
@@ -1198,18 +965,12 @@ func (s *GroupsService) ShareGroupWithGroup(gid any, opt *ShareGroupWithGroupOpt
 // GitLab API docs:
 // https://docs.gitlab.com/api/groups/#delete-the-link-that-shares-a-group-with-another-group
 func (s *GroupsService) UnshareGroupFromGroup(gid any, groupID int64, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf("groups/%s/share/%d", PathEscape(group), groupID)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s/share/%d", GroupID{gid}, groupID),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // GroupPushRules represents a group push rule.
@@ -1239,24 +1000,10 @@ type GroupPushRules struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_push_rules/#get-the-push-rules-of-a-group
 func (s *GroupsService) GetGroupPushRules(gid any, options ...RequestOptionFunc) (*GroupPushRules, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/push_rule", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	gpr := new(GroupPushRules)
-	resp, err := s.client.Do(req, gpr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gpr, resp, nil
+	return do[*GroupPushRules](s.client,
+		withPath("groups/%s/push_rule", GroupID{gid}),
+		withRequestOpts(options...),
+	)
 }
 
 // AddGroupPushRuleOptions represents the available AddGroupPushRule()
@@ -1285,24 +1032,12 @@ type AddGroupPushRuleOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_push_rules/#add-push-rules-to-a-group
 func (s *GroupsService) AddGroupPushRule(gid any, opt *AddGroupPushRuleOptions, options ...RequestOptionFunc) (*GroupPushRules, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/push_rule", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	gpr := new(GroupPushRules)
-	resp, err := s.client.Do(req, gpr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gpr, resp, nil
+	return do[*GroupPushRules](s.client,
+		withMethod(http.MethodPost),
+		withPath("groups/%s/push_rule", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // EditGroupPushRuleOptions represents the available EditGroupPushRule()
@@ -1331,24 +1066,12 @@ type EditGroupPushRuleOptions struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_push_rules/#edit-the-push-rules-of-a-group
 func (s *GroupsService) EditGroupPushRule(gid any, opt *EditGroupPushRuleOptions, options ...RequestOptionFunc) (*GroupPushRules, *Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("groups/%s/push_rule", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodPut, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	gpr := new(GroupPushRules)
-	resp, err := s.client.Do(req, gpr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return gpr, resp, nil
+	return do[*GroupPushRules](s.client,
+		withMethod(http.MethodPut),
+		withPath("groups/%s/push_rule", GroupID{gid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // DeleteGroupPushRule deletes the push rules of a group.
@@ -1356,16 +1079,10 @@ func (s *GroupsService) EditGroupPushRule(gid any, opt *EditGroupPushRuleOptions
 // GitLab API docs:
 // https://docs.gitlab.com/api/group_push_rules/#delete-the-push-rules-of-a-group
 func (s *GroupsService) DeleteGroupPushRule(gid any, options ...RequestOptionFunc) (*Response, error) {
-	group, err := parseID(gid)
-	if err != nil {
-		return nil, err
-	}
-	u := fmt.Sprintf("groups/%s/push_rule", PathEscape(group))
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("groups/%s/push_rule", GroupID{gid}),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
