@@ -461,18 +461,11 @@ type ListUsersOptions struct {
 }
 
 func (s *UsersService) ListUsers(opt *ListUsersOptions, options ...RequestOptionFunc) ([]*User, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "users", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var usr []*User
-	resp, err := s.client.Do(req, &usr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return usr, resp, nil
+	return do[[]*User](s.client,
+		withPath("users"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // GetUsersOptions represents the available GetUser() options.
@@ -483,20 +476,11 @@ type GetUsersOptions struct {
 }
 
 func (s *UsersService) GetUser(user int64, opt GetUsersOptions, options ...RequestOptionFunc) (*User, *Response, error) {
-	u := fmt.Sprintf("users/%d", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	usr := new(User)
-	resp, err := s.client.Do(req, usr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return usr, resp, nil
+	return do[*User](s.client,
+		withPath("users/%d", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // CreateUserOptions represents the available CreateUser() options.
@@ -528,6 +512,7 @@ type CreateUserOptions struct {
 	Twitter             *string     `url:"twitter,omitempty" json:"twitter,omitempty"`
 	Username            *string     `url:"username,omitempty" json:"username,omitempty"`
 	WebsiteURL          *string     `url:"website_url,omitempty" json:"website_url,omitempty"`
+	ViewDiffsFileByFile *bool       `url:"view_diffs_file_by_file,omitempty" json:"view_diffs_file_by_file,omitempty"`
 }
 
 func (s *UsersService) CreateUser(opt *CreateUserOptions, options ...RequestOptionFunc) (*User, *Response, error) {
@@ -564,31 +549,32 @@ func (s *UsersService) CreateUser(opt *CreateUserOptions, options ...RequestOpti
 //
 // GitLab API docs: https://docs.gitlab.com/api/users/#modify-a-user
 type ModifyUserOptions struct {
-	Admin              *bool       `url:"admin,omitempty" json:"admin,omitempty"`
-	Avatar             *UserAvatar `url:"-" json:"avatar,omitempty"`
-	Bio                *string     `url:"bio,omitempty" json:"bio,omitempty"`
-	CanCreateGroup     *bool       `url:"can_create_group,omitempty" json:"can_create_group,omitempty"`
-	CommitEmail        *string     `url:"commit_email,omitempty" json:"commit_email,omitempty"`
-	Email              *string     `url:"email,omitempty" json:"email,omitempty"`
-	External           *bool       `url:"external,omitempty" json:"external,omitempty"`
-	ExternUID          *string     `url:"extern_uid,omitempty" json:"extern_uid,omitempty"`
-	JobTitle           *string     `url:"job_title,omitempty" json:"job_title,omitempty"`
-	Linkedin           *string     `url:"linkedin,omitempty" json:"linkedin,omitempty"`
-	Location           *string     `url:"location,omitempty" json:"location,omitempty"`
-	Name               *string     `url:"name,omitempty" json:"name,omitempty"`
-	Note               *string     `url:"note,omitempty" json:"note,omitempty"`
-	Organization       *string     `url:"organization,omitempty" json:"organization,omitempty"`
-	Password           *string     `url:"password,omitempty" json:"password,omitempty"`
-	PrivateProfile     *bool       `url:"private_profile,omitempty" json:"private_profile,omitempty"`
-	ProjectsLimit      *int64      `url:"projects_limit,omitempty" json:"projects_limit,omitempty"`
-	Provider           *string     `url:"provider,omitempty" json:"provider,omitempty"`
-	PublicEmail        *string     `url:"public_email,omitempty" json:"public_email,omitempty"`
-	SkipReconfirmation *bool       `url:"skip_reconfirmation,omitempty" json:"skip_reconfirmation,omitempty"`
-	Skype              *string     `url:"skype,omitempty" json:"skype,omitempty"`
-	ThemeID            *int64      `url:"theme_id,omitempty" json:"theme_id,omitempty"`
-	Twitter            *string     `url:"twitter,omitempty" json:"twitter,omitempty"`
-	Username           *string     `url:"username,omitempty" json:"username,omitempty"`
-	WebsiteURL         *string     `url:"website_url,omitempty" json:"website_url,omitempty"`
+	Admin               *bool       `url:"admin,omitempty" json:"admin,omitempty"`
+	Avatar              *UserAvatar `url:"-" json:"avatar,omitempty"`
+	Bio                 *string     `url:"bio,omitempty" json:"bio,omitempty"`
+	CanCreateGroup      *bool       `url:"can_create_group,omitempty" json:"can_create_group,omitempty"`
+	CommitEmail         *string     `url:"commit_email,omitempty" json:"commit_email,omitempty"`
+	Email               *string     `url:"email,omitempty" json:"email,omitempty"`
+	External            *bool       `url:"external,omitempty" json:"external,omitempty"`
+	ExternUID           *string     `url:"extern_uid,omitempty" json:"extern_uid,omitempty"`
+	JobTitle            *string     `url:"job_title,omitempty" json:"job_title,omitempty"`
+	Linkedin            *string     `url:"linkedin,omitempty" json:"linkedin,omitempty"`
+	Location            *string     `url:"location,omitempty" json:"location,omitempty"`
+	Name                *string     `url:"name,omitempty" json:"name,omitempty"`
+	Note                *string     `url:"note,omitempty" json:"note,omitempty"`
+	Organization        *string     `url:"organization,omitempty" json:"organization,omitempty"`
+	Password            *string     `url:"password,omitempty" json:"password,omitempty"`
+	PrivateProfile      *bool       `url:"private_profile,omitempty" json:"private_profile,omitempty"`
+	ProjectsLimit       *int64      `url:"projects_limit,omitempty" json:"projects_limit,omitempty"`
+	Provider            *string     `url:"provider,omitempty" json:"provider,omitempty"`
+	PublicEmail         *string     `url:"public_email,omitempty" json:"public_email,omitempty"`
+	SkipReconfirmation  *bool       `url:"skip_reconfirmation,omitempty" json:"skip_reconfirmation,omitempty"`
+	Skype               *string     `url:"skype,omitempty" json:"skype,omitempty"`
+	ThemeID             *int64      `url:"theme_id,omitempty" json:"theme_id,omitempty"`
+	Twitter             *string     `url:"twitter,omitempty" json:"twitter,omitempty"`
+	Username            *string     `url:"username,omitempty" json:"username,omitempty"`
+	WebsiteURL          *string     `url:"website_url,omitempty" json:"website_url,omitempty"`
+	ViewDiffsFileByFile *bool       `url:"view_diffs_file_by_file,omitempty" json:"view_diffs_file_by_file,omitempty"`
 }
 
 func (s *UsersService) ModifyUser(user int64, opt *ModifyUserOptions, options ...RequestOptionFunc) (*User, *Response, error) {
@@ -623,29 +609,19 @@ func (s *UsersService) ModifyUser(user int64, opt *ModifyUserOptions, options ..
 }
 
 func (s *UsersService) DeleteUser(user int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("users/%d", user)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("users/%d", user),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 func (s *UsersService) CurrentUser(options ...RequestOptionFunc) (*User, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "user", nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	usr := new(User)
-	resp, err := s.client.Do(req, usr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return usr, resp, nil
+	return do[*User](s.client,
+		withPath("user"),
+		withRequestOpts(options...),
+	)
 }
 
 // UserStatus represents the current status of a user
@@ -661,18 +637,10 @@ type UserStatus struct {
 }
 
 func (s *UsersService) CurrentUserStatus(options ...RequestOptionFunc) (*UserStatus, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "user/status", nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	status := new(UserStatus)
-	resp, err := s.client.Do(req, status)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return status, resp, nil
+	return do[*UserStatus](s.client,
+		withPath("user/status"),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetUserStatus(uid any, options ...RequestOptionFunc) (*UserStatus, *Response, error) {
@@ -709,18 +677,12 @@ type UserStatusOptions struct {
 }
 
 func (s *UsersService) SetUserStatus(opt *UserStatusOptions, options ...RequestOptionFunc) (*UserStatus, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodPut, "user/status", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	status := new(UserStatus)
-	resp, err := s.client.Do(req, status)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return status, resp, nil
+	return do[*UserStatus](s.client,
+		withMethod(http.MethodPut),
+		withPath("user/status"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // UserAssociationsCount represents the user associations count.
@@ -739,20 +701,10 @@ type UserAssociationsCount struct {
 // GitLab API docs:
 // https://docs.gitlab.com/api/users/#get-a-count-of-a-users-projects-groups-issues-and-merge-requests
 func (s *UsersService) GetUserAssociationsCount(user int64, options ...RequestOptionFunc) (*UserAssociationsCount, *Response, error) {
-	u := fmt.Sprintf("users/%d/associations_count", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	uac := new(UserAssociationsCount)
-	resp, err := s.client.Do(req, uac)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return uac, resp, nil
+	return do[*UserAssociationsCount](s.client,
+		withPath("users/%d/associations_count", user),
+		withRequestOpts(options...),
+	)
 }
 
 // SSHKey represents a SSH key.
@@ -775,18 +727,11 @@ type ListSSHKeysOptions struct {
 }
 
 func (s *UsersService) ListSSHKeys(opt *ListSSHKeysOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "user/keys", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var k []*SSHKey
-	resp, err := s.client.Do(req, &k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[[]*SSHKey](s.client,
+		withPath("user/keys"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // ListSSHKeysForUserOptions represents the available ListSSHKeysForUser() options.
@@ -798,58 +743,25 @@ type ListSSHKeysForUserOptions struct {
 }
 
 func (s *UsersService) ListSSHKeysForUser(uid any, opt *ListSSHKeysForUserOptions, options ...RequestOptionFunc) ([]*SSHKey, *Response, error) {
-	user, err := parseID(uid)
-	if err != nil {
-		return nil, nil, err
-	}
-	u := fmt.Sprintf("users/%s/keys", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var k []*SSHKey
-	resp, err := s.client.Do(req, &k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[[]*SSHKey](s.client,
+		withPath("users/%s/keys", UserID{uid}),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetSSHKey(key int64, options ...RequestOptionFunc) (*SSHKey, *Response, error) {
-	u := fmt.Sprintf("user/keys/%d", key)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(SSHKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*SSHKey](s.client,
+		withPath("user/keys/%d", key),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetSSHKeyForUser(user int64, key int64, options ...RequestOptionFunc) (*SSHKey, *Response, error) {
-	u := fmt.Sprintf("users/%d/keys/%d", user, key)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(SSHKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*SSHKey](s.client,
+		withPath("users/%d/keys/%d", user, key),
+		withRequestOpts(options...),
+	)
 }
 
 // AddSSHKeyOptions represents the available AddSSHKey() options.
@@ -863,57 +775,39 @@ type AddSSHKeyOptions struct {
 }
 
 func (s *UsersService) AddSSHKey(opt *AddSSHKeyOptions, options ...RequestOptionFunc) (*SSHKey, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodPost, "user/keys", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(SSHKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*SSHKey](s.client,
+		withMethod(http.MethodPost),
+		withPath("user/keys"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) AddSSHKeyForUser(user int64, opt *AddSSHKeyOptions, options ...RequestOptionFunc) (*SSHKey, *Response, error) {
-	u := fmt.Sprintf("users/%d/keys", user)
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(SSHKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*SSHKey](s.client,
+		withMethod(http.MethodPost),
+		withPath("users/%d/keys", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) DeleteSSHKey(key int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("user/keys/%d", key)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("user/keys/%d", key),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 func (s *UsersService) DeleteSSHKeyForUser(user, key int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("users/%d/keys/%d", user, key)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("users/%d/keys/%d", user, key),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // GPGKey represents a GPG key.
@@ -926,35 +820,17 @@ type GPGKey struct {
 }
 
 func (s *UsersService) ListGPGKeys(options ...RequestOptionFunc) ([]*GPGKey, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "user/gpg_keys", nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var ks []*GPGKey
-	resp, err := s.client.Do(req, &ks)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return ks, resp, nil
+	return do[[]*GPGKey](s.client,
+		withPath("user/gpg_keys"),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetGPGKey(key int64, options ...RequestOptionFunc) (*GPGKey, *Response, error) {
-	u := fmt.Sprintf("user/gpg_keys/%d", key)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(GPGKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*GPGKey](s.client,
+		withPath("user/gpg_keys/%d", key),
+		withRequestOpts(options...),
+	)
 }
 
 // AddGPGKeyOptions represents the available AddGPGKey() options.
@@ -965,91 +841,53 @@ type AddGPGKeyOptions struct {
 }
 
 func (s *UsersService) AddGPGKey(opt *AddGPGKeyOptions, options ...RequestOptionFunc) (*GPGKey, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodPost, "user/gpg_keys", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(GPGKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*GPGKey](s.client,
+		withMethod(http.MethodPost),
+		withPath("user/gpg_keys"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) DeleteGPGKey(key int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("user/gpg_keys/%d", key)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("user/gpg_keys/%d", key),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 func (s *UsersService) ListGPGKeysForUser(user int64, options ...RequestOptionFunc) ([]*GPGKey, *Response, error) {
-	u := fmt.Sprintf("users/%d/gpg_keys", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var ks []*GPGKey
-	resp, err := s.client.Do(req, &ks)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return ks, resp, nil
+	return do[[]*GPGKey](s.client,
+		withPath("users/%d/gpg_keys", user),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetGPGKeyForUser(user, key int64, options ...RequestOptionFunc) (*GPGKey, *Response, error) {
-	u := fmt.Sprintf("users/%d/gpg_keys/%d", user, key)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(GPGKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*GPGKey](s.client,
+		withPath("users/%d/gpg_keys/%d", user, key),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) AddGPGKeyForUser(user int64, opt *AddGPGKeyOptions, options ...RequestOptionFunc) (*GPGKey, *Response, error) {
-	u := fmt.Sprintf("users/%d/gpg_keys", user)
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	k := new(GPGKey)
-	resp, err := s.client.Do(req, k)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return k, resp, nil
+	return do[*GPGKey](s.client,
+		withMethod(http.MethodPost),
+		withPath("users/%d/gpg_keys", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) DeleteGPGKeyForUser(user, key int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("users/%d/gpg_keys/%d", user, key)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("users/%d/gpg_keys/%d", user, key),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // Email represents an Email.
@@ -1063,18 +901,10 @@ type Email struct {
 }
 
 func (s *UsersService) ListEmails(options ...RequestOptionFunc) ([]*Email, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "user/emails", nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var e []*Email
-	resp, err := s.client.Do(req, &e)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return e, resp, nil
+	return do[[]*Email](s.client,
+		withPath("user/emails"),
+		withRequestOpts(options...),
+	)
 }
 
 // ListEmailsForUserOptions represents the available ListEmailsForUser() options.
@@ -1086,37 +916,18 @@ type ListEmailsForUserOptions struct {
 }
 
 func (s *UsersService) ListEmailsForUser(user int64, opt *ListEmailsForUserOptions, options ...RequestOptionFunc) ([]*Email, *Response, error) {
-	u := fmt.Sprintf("users/%d/emails", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var e []*Email
-	resp, err := s.client.Do(req, &e)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return e, resp, nil
+	return do[[]*Email](s.client,
+		withPath("users/%d/emails", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetEmail(email int64, options ...RequestOptionFunc) (*Email, *Response, error) {
-	u := fmt.Sprintf("user/emails/%d", email)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	e := new(Email)
-	resp, err := s.client.Do(req, e)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return e, resp, nil
+	return do[*Email](s.client,
+		withPath("user/emails/%d", email),
+		withRequestOpts(options...),
+	)
 }
 
 // AddEmailOptions represents the available AddEmail() options.
@@ -1129,57 +940,39 @@ type AddEmailOptions struct {
 }
 
 func (s *UsersService) AddEmail(opt *AddEmailOptions, options ...RequestOptionFunc) (*Email, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodPost, "user/emails", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	e := new(Email)
-	resp, err := s.client.Do(req, e)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return e, resp, nil
+	return do[*Email](s.client,
+		withMethod(http.MethodPost),
+		withPath("user/emails"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) AddEmailForUser(user int64, opt *AddEmailOptions, options ...RequestOptionFunc) (*Email, *Response, error) {
-	u := fmt.Sprintf("users/%d/emails", user)
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	e := new(Email)
-	resp, err := s.client.Do(req, e)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return e, resp, nil
+	return do[*Email](s.client,
+		withMethod(http.MethodPost),
+		withPath("users/%d/emails", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) DeleteEmail(email int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("user/emails/%d", email)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("user/emails/%d", email),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 func (s *UsersService) DeleteEmailForUser(user, email int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("users/%d/emails/%d", user, email)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("users/%d/emails/%d", user, email),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 func (s *UsersService) BlockUser(user int64, options ...RequestOptionFunc) error {
@@ -1398,37 +1191,18 @@ type GetAllImpersonationTokensOptions struct {
 }
 
 func (s *UsersService) GetAllImpersonationTokens(user int64, opt *GetAllImpersonationTokensOptions, options ...RequestOptionFunc) ([]*ImpersonationToken, *Response, error) {
-	u := fmt.Sprintf("users/%d/impersonation_tokens", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var ts []*ImpersonationToken
-	resp, err := s.client.Do(req, &ts)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return ts, resp, nil
+	return do[[]*ImpersonationToken](s.client,
+		withPath("users/%d/impersonation_tokens", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) GetImpersonationToken(user, token int64, options ...RequestOptionFunc) (*ImpersonationToken, *Response, error) {
-	u := fmt.Sprintf("users/%d/impersonation_tokens/%d", user, token)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, nil, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	t := new(ImpersonationToken)
-	resp, err := s.client.Do(req, &t)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return t, resp, nil
+	return do[*ImpersonationToken](s.client,
+		withPath("users/%d/impersonation_tokens/%d", user, token),
+		withRequestOpts(options...),
+	)
 }
 
 // CreateImpersonationTokenOptions represents the available
@@ -1443,31 +1217,21 @@ type CreateImpersonationTokenOptions struct {
 }
 
 func (s *UsersService) CreateImpersonationToken(user int64, opt *CreateImpersonationTokenOptions, options ...RequestOptionFunc) (*ImpersonationToken, *Response, error) {
-	u := fmt.Sprintf("users/%d/impersonation_tokens", user)
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	t := new(ImpersonationToken)
-	resp, err := s.client.Do(req, &t)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return t, resp, nil
+	return do[*ImpersonationToken](s.client,
+		withMethod(http.MethodPost),
+		withPath("users/%d/impersonation_tokens", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) RevokeImpersonationToken(user, token int64, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("users/%d/impersonation_tokens/%d", user, token)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("users/%d/impersonation_tokens/%d", user, token),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
 
 // CreatePersonalAccessTokenOptions represents the available
@@ -1483,20 +1247,12 @@ type CreatePersonalAccessTokenOptions struct {
 }
 
 func (s *UsersService) CreatePersonalAccessToken(user int64, opt *CreatePersonalAccessTokenOptions, options ...RequestOptionFunc) (*PersonalAccessToken, *Response, error) {
-	u := fmt.Sprintf("users/%d/personal_access_tokens", user)
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	t := new(PersonalAccessToken)
-	resp, err := s.client.Do(req, &t)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return t, resp, nil
+	return do[*PersonalAccessToken](s.client,
+		withMethod(http.MethodPost),
+		withPath("users/%d/personal_access_tokens", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // CreatePersonalAccessTokenForCurrentUserOptions represents the available
@@ -1512,20 +1268,12 @@ type CreatePersonalAccessTokenForCurrentUserOptions struct {
 }
 
 func (s *UsersService) CreatePersonalAccessTokenForCurrentUser(opt *CreatePersonalAccessTokenForCurrentUserOptions, options ...RequestOptionFunc) (*PersonalAccessToken, *Response, error) {
-	u := "user/personal_access_tokens"
-
-	req, err := s.client.NewRequest(http.MethodPost, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	t := new(PersonalAccessToken)
-	resp, err := s.client.Do(req, &t)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return t, resp, nil
+	return do[*PersonalAccessToken](s.client,
+		withMethod(http.MethodPost),
+		withPath("user/personal_access_tokens"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // UserActivity represents an entry in the user/activities response
@@ -1547,18 +1295,11 @@ type GetUserActivitiesOptions struct {
 }
 
 func (s *UsersService) GetUserActivities(opt *GetUserActivitiesOptions, options ...RequestOptionFunc) ([]*UserActivity, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "user/activities", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var t []*UserActivity
-	resp, err := s.client.Do(req, &t)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return t, resp, nil
+	return do[[]*UserActivity](s.client,
+		withPath("user/activities"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 // UserMembership represents a membership of the user in a namespace or project.
@@ -1582,20 +1323,11 @@ type GetUserMembershipOptions struct {
 }
 
 func (s *UsersService) GetUserMemberships(user int64, opt *GetUserMembershipOptions, options ...RequestOptionFunc) ([]*UserMembership, *Response, error) {
-	u := fmt.Sprintf("users/%d/memberships", user)
-
-	req, err := s.client.NewRequest(http.MethodGet, u, opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var m []*UserMembership
-	resp, err := s.client.Do(req, &m)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return m, resp, nil
+	return do[[]*UserMembership](s.client,
+		withPath("users/%d/memberships", user),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) DisableTwoFactor(user int64, options ...RequestOptionFunc) error {
@@ -1654,18 +1386,12 @@ type CreateUserRunnerOptions struct {
 }
 
 func (s *UsersService) CreateUserRunner(opts *CreateUserRunnerOptions, options ...RequestOptionFunc) (*UserRunner, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodPost, "user/runners", opts, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	r := new(UserRunner)
-	resp, err := s.client.Do(req, r)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return r, resp, nil
+	return do[*UserRunner](s.client,
+		withMethod(http.MethodPost),
+		withPath("user/runners"),
+		withAPIOpts(opts),
+		withRequestOpts(options...),
+	)
 }
 
 // CreateServiceAccountUserOptions represents the available CreateServiceAccountUser() options.
@@ -1679,33 +1405,20 @@ type CreateServiceAccountUserOptions struct {
 }
 
 func (s *UsersService) CreateServiceAccountUser(opts *CreateServiceAccountUserOptions, options ...RequestOptionFunc) (*User, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodPost, "service_accounts", opts, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	usr := new(User)
-	resp, err := s.client.Do(req, usr)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return usr, resp, nil
+	return do[*User](s.client,
+		withMethod(http.MethodPost),
+		withPath("service_accounts"),
+		withAPIOpts(opts),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) ListServiceAccounts(opt *ListServiceAccountsOptions, options ...RequestOptionFunc) ([]*ServiceAccount, *Response, error) {
-	req, err := s.client.NewRequest(http.MethodGet, "service_accounts", opt, options)
-	if err != nil {
-		return nil, nil, err
-	}
-
-	var sas []*ServiceAccount
-	resp, err := s.client.Do(req, &sas)
-	if err != nil {
-		return nil, resp, err
-	}
-
-	return sas, resp, nil
+	return do[[]*ServiceAccount](s.client,
+		withPath("service_accounts"),
+		withAPIOpts(opt),
+		withRequestOpts(options...),
+	)
 }
 
 func (s *UsersService) UploadAvatar(avatar io.Reader, filename string, options ...RequestOptionFunc) (*User, *Response, error) {
@@ -1734,12 +1447,10 @@ func (s *UsersService) UploadAvatar(avatar io.Reader, filename string, options .
 }
 
 func (s *UsersService) DeleteUserIdentity(user int64, provider string, options ...RequestOptionFunc) (*Response, error) {
-	u := fmt.Sprintf("users/%d/identities/%s", user, provider)
-
-	req, err := s.client.NewRequest(http.MethodDelete, u, nil, options)
-	if err != nil {
-		return nil, err
-	}
-
-	return s.client.Do(req, nil)
+	_, resp, err := do[none](s.client,
+		withMethod(http.MethodDelete),
+		withPath("users/%d/identities/%s", user, provider),
+		withRequestOpts(options...),
+	)
+	return resp, err
 }
